import java.awt.Container;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.LinkedBlockingQueue;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextArea;

/**
 * 
 * GUI for when client is logged in.
 *
 */
public class MainFrame extends JFrame {
	LinkedBlockingQueue<Msg> inbox;

	String msg1 = "", msg2 = "";

	JLabel autoSyncLabel1 = new JLabel();
	JLabel autoSyncLabel2 = new JLabel();
	JButton autoSyncButton = new JButton();
	JLabel userNamelabel = new JLabel();
	JTextArea output = new JTextArea();
	JButton syncUp = new JButton("Sync Up");
	JButton syncDown = new JButton("Sync Down");
	Container container;

	int frameWidth = 500;
	int marginTop = 20;
	int marginBottom = 50;
	int marginRight = 10;
	int verticalGap = 10;
	int horizontalGap = 10;

	int autoSyncLabel1Width = 200;
	int autoSyncLabel1Height = 20;
	int autoSyncLabel2Width = 200;
	int autoSyncLabel2Height = 20;
	int autoSyncButtonWidth = 200;
	int autoSyncButtonHeight = 30;
	int userNameLabelWidth = 450;
	int userNameLabelHeight = 40;
	int outputWidth = 450;
	int outputHeight = 200;
	int syncUpWidth = 200;
	int syncUpHeight = 50;
	int syncDownWidth = 200;
	int syncDownHeight = 50;

	MainFrame(LinkedBlockingQueue<Msg> inbox, String m1, String m2, String userNameLabelText) {
		this.inbox = inbox;
		if (m1 != null)
			msg1 = m1;
		if (m2 != null)
			msg2 = m2;
		container = getContentPane();
		container.setLayout(null);

		int y = marginTop;

		autoSyncLabel1.setFont(new Font(null, Font.PLAIN, 15));
		autoSyncLabel1.setBounds(frameWidth - marginRight - autoSyncLabel1Width, y, autoSyncLabel1Width,
				autoSyncLabel1Height);
		container.add(autoSyncLabel1);
		y += autoSyncLabel1Height + 3;

		autoSyncLabel2.setFont(new Font(null, Font.PLAIN, 15));
		autoSyncLabel2.setBounds(frameWidth - marginRight - autoSyncLabel2Width, y, autoSyncLabel2Width,
				autoSyncLabel2Height);
		container.add(autoSyncLabel2);
		y += autoSyncLabel2Height + 3;

		autoSyncButton.setBounds(frameWidth - marginRight - autoSyncButtonWidth, y, autoSyncButtonWidth,
				autoSyncButtonHeight);
		container.add(autoSyncButton);
		y += autoSyncButtonHeight + 3;

		userNamelabel.setText(userNameLabelText);
		userNamelabel.setFont(new Font(null, Font.PLAIN, 15));
		userNamelabel.setBounds((frameWidth - userNameLabelWidth) / 2, y, userNameLabelWidth, userNameLabelHeight);
		container.add(userNamelabel);
		y += userNameLabelHeight + verticalGap;

		output.setFont(new Font(null, Font.PLAIN, 20));
		output.setEditable(false);
		output.setText(msg1 + "\n" + msg2);
		output.setLineWrap(true);
		output.setMargin(new Insets(10, 10, 10, 10));
		output.setBounds((frameWidth - outputWidth) / 2, y, outputWidth, outputHeight);
		container.add(output);
		y += outputHeight + verticalGap;

		int x = (frameWidth - (syncUpWidth + horizontalGap + syncDownWidth)) / 2;
		syncUp.setBounds(x, y, syncUpWidth, syncUpHeight);
		container.add(syncUp);
		x += syncUpWidth + horizontalGap;

		syncDown.setBounds(x, y, syncDownWidth, syncDownHeight);
		container.add(syncDown);
		y += syncDownHeight + verticalGap;

		y += marginBottom;
		setSize(frameWidth, y);
		setTitle("FSync");

		syncUp.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Msg m = new Msg(Msg.SYNC_UP_ALARM, null);
				inbox.add(m);
			}
		});

		syncDown.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Msg m = new Msg(Msg.SYNC_DOWN_ALARM, null);
				inbox.add(m);
			}
		});

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);
	}

	/**
	 * Update the output text.
	 * 
	 * @param m1 : line1 text, if null then line1 text is unchanged.
	 * @param m2 : line2 text, if null then line2 text is unchanged.
	 */
	public synchronized void updateOutput(String m1, String m2) {
		if (m1 != null)
			msg1 = m1;
		if (m2 != null)
			msg2 = m2;
		output.setText(msg1 + "\n" + msg2);
		revalidate();
	}

	/**
	 * Update the auto sync status.
	 * 
	 * @param isOn     : is auto sync on.
	 * @param interval : auto sync time interval
	 */
	public synchronized void updateAutoSyncStatus(boolean isOn, int interval) {
		if (isOn) {
			autoSyncLabel1.setText("Auto Sync : On");
			autoSyncLabel2.setText("Every " + interval + " seconds");
			autoSyncButton.setText("Turn Off");
			for (ActionListener al : autoSyncButton.getActionListeners()) {
				autoSyncButton.removeActionListener(al);
			}
			autoSyncButton.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					Msg m = new Msg(Msg.TURN_OFF_AUTO_SYNC, null);
					inbox.add(m);
				}
			});
		} else {
			autoSyncLabel1.setText("Auto Sync : Off");
			autoSyncLabel2.setText("");
			autoSyncButton.setText("Turn On");
			for (ActionListener al : autoSyncButton.getActionListeners()) {
				autoSyncButton.removeActionListener(al);
			}
			autoSyncButton.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					SetIntervalFrame sif = new SetIntervalFrame(inbox);
					sif.setLocation(getX(), getY() + (getHeight() - sif.getHeight()) / 2);
					sif.setVisible(true);
				}
			});
		}
		revalidate();
	}
}
