import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * 
 * Represents the server socket used to accept client connections.
 *
 */
public class ListeningSocket {
	int port;
	LinkedBlockingQueue<Msg> inbox;
	ServerSocket serverSocket = null;

	/**
	 * @param inbox : inbox of main thread
	 */
	public ListeningSocket(LinkedBlockingQueue<Msg> inbox) {
		this.inbox = inbox;
		port = 0;
	}

	/**
	 * 
	 * @param port  : port number on which to listen for new connections
	 * @param inbox : inbox of main thread
	 */
	public ListeningSocket(int port, LinkedBlockingQueue<Msg> inbox) {
		this.inbox = inbox;
		this.port = port;
	}

	/**
	 * Creates a server socket and starts a new thread to accept the incoming
	 * connections.
	 * 
	 * @throws IOException
	 */
	public void start() throws IOException {
		serverSocket = new ServerSocket(port);
		new Thread(new Runnable() {

			@Override
			public void run() {
				while (true) {
					try {
						Socket socket = serverSocket.accept();
						Connection connection = new Connection(socket, inbox);
						connection.startRecSendThreads();
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}).start();
	}

	/**
	 * 
	 * @return port number on which socket is running or 0 if the socket is not
	 *         running
	 */
	public int getPort() {
		if (serverSocket != null)
			return serverSocket.getLocalPort();
		return 0;
	}

	/**
	 * @return local address of the socket
	 */
	public String getSocketAddr() {
		if (serverSocket != null)
			return serverSocket.getLocalSocketAddress().toString();
		return null;
	}
}