/**
 * 
 * Contains the rolling hash of a sequence of bytes.
 *
 */
public class RollingHash {
	private final int X = 31;
	private int XtoN;
	private int hash;

	/**
	 * @param blockSize : size of byte sequence
	 */
	public RollingHash(int blockSize) {
		XtoN = 1;
		while (blockSize-- > 0)
			XtoN *= X;
	}

	/**
	 * @return : current value of hash
	 */
	public int getHash() {
		return hash;
	}

	/**
	 * Resets the hash value to 0.
	 */
	public void reset() {
		hash = 0;
	}

	/**
	 * Add the given byte to the sequence and update the hash.
	 * 
	 * @param b : byte to be added
	 */
	public void update(byte b) {
		hash = X * hash + b;
	}

	/**
	 * Add one byte and delete another from the sequence and update the hash.
	 * 
	 * @param inByte  : byte to be added
	 * @param outByte : byte to be deleted
	 */
	public void update(byte inByte, byte outByte) {
		hash = X * hash + inByte - XtoN * outByte;
	}

	/**
	 * Add len number of bytes from arr starting from offset to the sequence and
	 * update the hash.
	 * 
	 * @param arr    : array containing bytes
	 * @param offset : starting position
	 * @param len    : number of bytes to add
	 */
	public void update(byte[] arr, int offset, int len) {
		for (int i = offset; i < offset + len; i++) {
			hash = X * hash + arr[i];
		}
	}
}
